#####################################################
# Simulate survival times under different scenarios #
#####################################################

# Simulate survival times when PH holds #
sim.T <- function(n, lambda, k){
  u <- runif(n)
  T <- (-log(u)/lambda)^(1/k)*12
  return(T)
}

# Simulate survival times when PH violates #
sim.T.var <- function(n, lam0, HR1, HR2, HR3, HR4, HR4_, k){
  T <- rep(0,n)
  t1 <- 4 /12
  t2 <- 8 /12
  t3 <- 12/12
  t4 <- 16/12
  lam1  <- lam0 * HR1 
  lam2  <- lam0 * HR2 
  lam3  <- lam0 * HR3
  lam4  <- lam0 * HR4 
  lam4_ <- lam0 * HR4_
  
  for (i in 1:n){
  u <- runif(1)
   if (u >= exp(-lam1*t1^k)) { T[i] <- (-log(u)/lam1)^(1/k) }
   else if (u < exp(-lam1*t1^k) & u >= exp(-lam1*t1^k-lam2*(t2^k-t1^k))) {
T[i] <- ((-log(u)-lam1*t1^k)/lam2+t1^k)^(1/k) } 
   else if (u < exp(-lam1*t1^k-lam2*(t2^k-t1^k)) & u >= exp(-lam1*t1^k-lam2*(t2^k-t1^k)-lam3*(t3^k-t2^k))) {
T[i] <- ((-log(u)-lam1*t1^k-lam2*(t2^k-t1^k))/lam3+t2^k)^(1/k) } 
   else if (u < exp(-lam1*t1^k-lam2*(t2^k-t1^k)-lam3*(t3^k-t2^k)) & u >= exp(-lam1*t1^k-lam2*(t2^k-t1^k)-lam3*(t3^k-t2^k)-lam4*(t4^k-t3^k))) { 
T[i] <- ((-log(u)-lam1*t1^k-lam2*(t2^k-t1^k)-lam3*(t3^k-t2^k))/lam4+t3^k)^(1/k) } 
   else { 
T[i] <- ((-log(u)-lam1*t1^k-lam2*(t2^k-t1^k)-lam3*(t3^k-t2^k)-lam4*(t4^k-t3^k))/lam4_+t4^k)^(1/k) } 
  }
  return(T*12)
}

######################################
# Caculate the censoring probability #
# using the Kaplan-Meier estimator   #
######################################

cal_KM <- function(X, status, X_new){
  tmpfit <- survfit(Surv(X, status) ~ 1)
  S <- summary(tmpfit, times = X_new)$surv
  S_new <- S[rank(X_new)]
  return(S_new)
}

#################################################
# Caculate the empirical likelihood probability #
# and the empirical likelihood ratio statistic  #
#################################################

cal.weight.JEL <- function(g.hat){
  M <- nrow(g.hat)
  g.col <- ncol(g.hat)
  Fn <- function(rho, ghat){ -sum(log(1 + ghat %*% rho)) }
  Grd <- function(rho, ghat){ -colSums(ghat / c(1 + ghat %*% rho)) }
  # calculate the weights
  rho.hat <- constrOptim(theta = rep(0, g.col), f = Fn, grad = Grd, ui = g.hat, ci = rep(1 / M - 1, M), ghat = g.hat)$par
  wts <- c(1 / M / (1 + g.hat %*% rho.hat))
  wts <- wts / sum(wts)
  return(list(wts = wts, ELR = -2 * sum(log(M * wts))))
}

##########################################
# Create the jackknife pseudo values for # 
# different restricted survival benefit  #
##########################################

# Restricted mean survival: E(min(T1,\tau) - min(T0,\tau)) #
Jack.RMST1 <- function(x,Y1,Y0,p1,p0,d1,d0){
  n1 <- length(Y1)
  n0 <- length(Y0)
  n1/(n1 * n0) * sum(apply(matrix(Y1,1,),2,"-",Y0)*apply(matrix(d1/p1,1,),2,"*",d0/p0)) - 
    (n1 - 1)/((n1 - 1) * n0) * sum(apply(matrix(Y1[-x],1,),2,"-",Y0)*apply(matrix(d1[-x]/p1[-x],1,),2,"*",d0/p0)) 
}

Jack.RMST2 <- function(x,Y1,Y0,p1,p0,d1,d0){
  n1 <- length(Y1)
  n0 <- length(Y0)
  n0/(n1 * n0) * sum(apply(matrix(Y1,1,),2,"-",Y0)*apply(matrix(d1/p1,1,),2,"*",d0/p0)) - 
    (n0 - 1)/(n1 * (n0 - 1)) * sum(apply(matrix(Y1,1,),2,"-",Y0[-x])*apply(matrix(d1/p1,1,),2,"*",d0[-x]/p0[-x])) 
}

Jack.RMST <- function(Y1,Y0,p1,p0,d1,d0){
  n1 <- length(Y1)
  n0 <- length(Y0)
  n <- n1 + n0
  U <- 1/(n1 * n0) * sum(apply(matrix(Y1,1,),2,"-",Y0)*apply(matrix(d1/p1,1,),2,"*",d0/p0)) 

  J1 <- sapply(1:length(Y1),FUN=Jack.RMST1,Y1=Y1,Y0=Y0,p1=p1,p0=p0,d1=d1,d0=d0)
  J2 <- sapply(1:length(Y0),FUN=Jack.RMST2,Y1=Y1,Y0=Y0,p1=p1,p0=p0,d1=d1,d0=d0)
  V1 <- -n/(n-2)*U + n/(n-2)*(n-1)*J1/n1
  V2 <- -n/(n-2)*U + n/(n-2)*(n-1)*J2/n0
  return(c(V1,V2))
}

# Restricted change of longer survival: P(min(T1,\tau) > m * min(T0,\tau)) #
Jack.ratio1 <- function(x,Y1,Y0,p1,p0,d1,d0,m){
  n1 <- length(Y1)
  n0 <- length(Y0)
  n1/(n1 * n0) * sum(apply(matrix(Y1,1,),2,">",m*Y0)*apply(matrix(d1/p1,1,),2,"*",d0/p0)) - 
    (n1 - 1)/((n1 - 1) * n0) * sum(apply(matrix(Y1[-x],1,),2,">",m*Y0)*apply(matrix(d1[-x]/p1[-x],1,),2,"*",d0/p0)) 
}

Jack.ratio2 <- function(x,Y1,Y0,p1,p0,d1,d0,m){
  n1 <- length(Y1)
  n0 <- length(Y0)
  n0/(n1 * n0) * sum(apply(matrix(Y1,1,),2,">",m*Y0)*apply(matrix(d1/p1,1,),2,"*",d0/p0)) - 
    (n0 - 1)/(n1 * (n0 - 1)) * sum(apply(matrix(Y1,1,),2,">",m*Y0[-x])*apply(matrix(d1/p1,1,),2,"*",d0[-x]/p0[-x])) 
}

Jack.ratio <- function(Y1,Y0,p1,p0,d1,d0,m){
  n1 <- length(Y1)
  n0 <- length(Y0)
  n <- n1 + n0
  U <- 1/(n1 * n0) * sum(apply(matrix(Y1,1,),2,">",m*Y0)*apply(matrix(d1/p1,1,),2,"*",d0/p0)) 

  J1 <- sapply(1:length(Y1),FUN=Jack.ratio1,Y1=Y1,Y0=Y0,p1=p1,p0=p0,d1=d1,d0=d0,m=m)
  J2 <- sapply(1:length(Y0),FUN=Jack.ratio2,Y1=Y1,Y0=Y0,p1=p1,p0=p0,d1=d1,d0=d0,m=m)
  V1 <- -n/(n-2)*U + n/(n-2)*(n-1)*J1/n1
  V2 <- -n/(n-2)*U + n/(n-2)*(n-1)*J2/n0
  return(c(V1,V2))
}

# Restricted change of longer survival: P(min(T1,\tau) > m + min(T0,\tau)) #
Jack.dif1 <- function(x,Y1,Y0,p1,p0,d1,d0,m){
  n1 <- length(Y1)
  n0 <- length(Y0)
  n1/(n1 * n0) * sum(apply(matrix(Y1,1,),2,">",m+Y0)*apply(matrix(d1/p1,1,),2,"*",d0/p0)) - 
    (n1 - 1)/((n1 - 1) * n0) * sum(apply(matrix(Y1[-x],1,),2,">",m+Y0)*apply(matrix(d1[-x]/p1[-x],1,),2,"*",d0/p0)) 
}

Jack.dif2 <- function(x,Y1,Y0,p1,p0,d1,d0,m){
  n1 <- length(Y1)
  n0 <- length(Y0)
  n0/(n1 * n0) * sum(apply(matrix(Y1,1,),2,">",m+Y0)*apply(matrix(d1/p1,1,),2,"*",d0/p0)) - 
    (n0 - 1)/(n1 * (n0 - 1)) * sum(apply(matrix(Y1,1,),2,">",m+Y0[-x])*apply(matrix(d1/p1,1,),2,"*",d0[-x]/p0[-x]))
}

Jack.dif <- function(Y1,Y0,p1,p0,d1,d0,m){
  n1 <- length(Y1)
  n0 <- length(Y0)
  n <- n1 + n0
  U <- 1/(n1 * n0) * sum(apply(matrix(Y1,1,),2,">",m+Y0)*apply(matrix(d1/p1,1,),2,"*",d0/p0))

  J1 <- sapply(1:length(Y1),FUN=Jack.dif1,Y1=Y1,Y0=Y0,p1=p1,p0=p0,d1=d1,d0=d0,m=m)
  J2 <- sapply(1:length(Y0),FUN=Jack.dif2,Y1=Y1,Y0=Y0,p1=p1,p0=p0,d1=d1,d0=d0,m=m)
  V1 <- -n/(n-2)*U + n/(n-2)*(n-1)*J1/n1
  V2 <- -n/(n-2)*U + n/(n-2)*(n-1)*J2/n0
  return(c(V1,V2))
}

# Restricted net change of longer survival:                                #
# P(min(T1,\tau) > m + min(T0,\tau)) - P(min(T0,\tau) > m + min(T1,\tau))  #
Jack.PP1 <- function(x,Y1,Y0,p1,p0,d1,d0,m){
  n1 <- length(Y1)
  n0 <- length(Y0)
  n1/(n1 * n0) * sum((apply(matrix(Y1,1,),2,">",m+Y0)-t(apply(matrix(Y0,1,),2,">",m+Y1)))*apply(matrix(d1/p1,1,),2,"*",d0/p0)) - 
    (n1 - 1)/((n1 - 1) * n0) * sum((apply(matrix(Y1[-x],1,),2,">",m+Y0)-t(apply(matrix(Y0,1,),2,">",m+Y1[-x])))*apply(matrix(d1[-x]/p1[-x],1,),2,"*",d0/p0)) 
}

Jack.PP2 <- function(x,Y1,Y0,p1,p0,d1,d0,m){
  n1 <- length(Y1)
  n0 <- length(Y0)
  n0/(n1 * n0) * sum((apply(matrix(Y1,1,),2,">",m+Y0)-t(apply(matrix(Y0,1,),2,">",m+Y1)))*apply(matrix(d1/p1,1,),2,"*",d0/p0)) - 
    (n0 - 1)/(n1 * (n0 - 1)) * sum((apply(matrix(Y1,1,),2,">",m+Y0[-x])-t(apply(matrix(Y0[-x],1,),2,">",m+Y1)))*apply(matrix(d1/p1,1,),2,"*",d0[-x]/p0[-x]))
}

Jack.PP <- function(Y1,Y0,p1,p0,d1,d0,m){
  n1 <- length(Y1)
  n0 <- length(Y0)
  n <- n1 + n0
  U <- 1/(n1 * n0) * sum((apply(matrix(Y1,1,),2,">",m+Y0)-t(apply(matrix(Y0,1,),2,">",m+Y1)))*apply(matrix(d1/p1,1,),2,"*",d0/p0))

  J1 <- sapply(1:length(Y1),FUN=Jack.PP1,Y1=Y1,Y0=Y0,p1=p1,p0=p0,d1=d1,d0=d0,m=m)
  J2 <- sapply(1:length(Y0),FUN=Jack.PP2,Y1=Y1,Y0=Y0,p1=p1,p0=p0,d1=d1,d0=d0,m=m)
  V1 <- -n/(n-2)*U + n/(n-2)*(n-1)*J1/n1
  V2 <- -n/(n-2)*U + n/(n-2)*(n-1)*J2/n0
  return(c(V1,V2))
}

###############################################
# Find empirical likelihood confidence        #
# intervals for different restricted survival #
# benefit using a bisection method            #
###############################################

# Restricted mean survival: E(min(T1,\tau) - min(T0,\tau)) #
findUL.RMST <- function(maxit = 30, Y1, Y0, p1, p0, d1, d0, MLE, level, del0, tol = 0.001){
n1 <- length(Y1)
n0 <- length(Y0)
n <- n1 + n0
J.vec <- Jack.RMST(Y1 = Y1, Y0 = Y0, p1 = p1, p0 = p0, d1 = d1, d0 = d0)
Upper.ini = MLE + 10
g.vec <-  J.vec - c(rep((n0-1)/n1,n1),rep((n1-1)/n0,n0)) * n/(n-2) * Upper.ini
ELRtest.tmp <- cal.weight.JEL(matrix(g.vec,,1))$ELR
while (ELRtest.tmp <= level) {
  Upper.ini = Upper.ini + 10
  g.vec <- J.vec - c(rep((n0-1)/n1,n1),rep((n1-1)/n0,n0)) * n/(n-2) * Upper.ini
  ELRtest.tmp <- cal.weight.JEL(matrix(g.vec,,1))$ELR
}
Lower.ini = MLE - 10
g.vec <- J.vec - c(rep((n0-1)/n1,n1),rep((n1-1)/n0,n0)) * n/(n-2) * Lower.ini
ELRtest.tmp <- cal.weight.JEL(matrix(g.vec,,1))$ELR
while (ELRtest.tmp <= level) {
  Lower.ini = Lower.ini - 10
  g.vec <- J.vec - c(rep((n0-1)/n1,n1),rep((n1-1)/n0,n0)) * n/(n-2) * Lower.ini
  ELRtest.tmp <- cal.weight.JEL(matrix(g.vec,,1))$ELR
}

# Upper limit
iter <- 0
dif <- 1
t1 <- MLE
t2 <- Upper.ini 
while(dif>tol & iter<maxit){
  tavg <- (t1+t2)/2
  g.vec <- J.vec - c(rep((n0-1)/n1,n1),rep((n1-1)/n0,n0)) * n/(n-2) * tavg
  elratio <- cal.weight.JEL(matrix(g.vec,,1))$ELR
  if(elratio > level) t2 <- tavg
  if(elratio <= level) t1 <- tavg
  dif <- t2-t1
  iter <- iter+1
}
UB <- (t1+t2)/2

# Lower limit
iter <- 0
dif <- 1
t1 <- MLE
t2 <- Lower.ini 
while(dif>tol & iter<maxit){
  tavg <- (t1+t2)/2
  g.vec <- J.vec - c(rep((n0-1)/n1,n1),rep((n1-1)/n0,n0)) * n/(n-2) * tavg
  elratio <- cal.weight.JEL(matrix(g.vec,,1))$ELR
  if(elratio > level) t2 <- tavg
  if(elratio <= level) t1 <- tavg
  dif <- t1-t2
  iter <- iter+1
}
LB <- (t1+t2)/2

cover <- ((LB<del0)&(UB>del0))
return(cover)
}

# Restricted change of longer survival: P(min(T1,\tau) > m * min(T0,\tau)) #
findUL.ratio <- function(maxit = 30, Lower.ini = 0, Upper.ini = 1, Y1, Y0, p1, p0, d1, d0, m = 1.1, MLE, level, del0, tol = 0.001){
n1 <- length(Y1)
n0 <- length(Y0)
n <- n1 + n0
J.vec <- Jack.ratio(Y1 = Y1, Y0 = Y0, p1 = p1, p0 = p0, d1 = d1, d0 = d0, m = m)
g.vec <- J.vec - c(rep((n0-1)/n1,n1),rep((n1-1)/n0,n0)) * n/(n-2) * Upper.ini
ELRtest.tmp <- cal.weight.JEL(matrix(g.vec,,1))$ELR
if (ELRtest.tmp <= level) stop()
g.vec <- J.vec - c(rep((n0-1)/n1,n1),rep((n1-1)/n0,n0)) * n/(n-2) * Lower.ini
ELRtest.tmp <- cal.weight.JEL(matrix(g.vec,,1))$ELR
if (ELRtest.tmp <= level) stop()

# Upper limit
iter <- 0
dif <- 1
t1 <- MLE
t2 <- Upper.ini 
while(dif>tol & iter<maxit){
  tavg <- (t1+t2)/2
  g.vec <- J.vec - c(rep((n0-1)/n1,n1),rep((n1-1)/n0,n0)) * n/(n-2) * tavg
  elratio <- cal.weight.JEL(matrix(g.vec,,1))$ELR
  if(elratio > level) t2 <- tavg
  if(elratio <= level) t1 <- tavg
  dif <- t2-t1
  iter <- iter+1
}
UB <- (t1+t2)/2

# Lower limit
iter <- 0
dif <- 1
t1 <- MLE
t2 <- Lower.ini 
while(dif>tol & iter<maxit){
  tavg <- (t1+t2)/2
  g.vec <- J.vec - c(rep((n0-1)/n1,n1),rep((n1-1)/n0,n0)) * n/(n-2) * tavg
  elratio <- cal.weight.JEL(matrix(g.vec,,1))$ELR
  if(elratio > level) t2 <- tavg
  if(elratio <= level) t1 <- tavg
  dif <- t1-t2
  iter <- iter+1
}
LB <- (t1+t2)/2

cover <- ((LB<del0)&(UB>del0))
return(cover)
}

# Restricted change of longer survival: P(min(T1,\tau) > m + min(T0,\tau)) #
findUL.dif <- function(maxit = 30, Lower.ini = 0, Upper.ini = 1, Y1, Y0, p1, p0, d1, d0, m = 0, MLE, level, del0, tol = 0.001){
n1 <- length(Y1)
n0 <- length(Y0)
n <- n1 + n0
J.vec <- Jack.dif(Y1 = Y1, Y0 = Y0, p1 = p1, p0 = p0, d1 = d1, d0 = d0, m = m)
g.vec <- J.vec - c(rep((n0-1)/n1,n1),rep((n1-1)/n0,n0)) * n/(n-2) * Upper.ini
ELRtest.tmp <- cal.weight.JEL(matrix(g.vec,,1))$ELR
if (ELRtest.tmp <= level) stop()
g.vec <- J.vec - c(rep((n0-1)/n1,n1),rep((n1-1)/n0,n0)) * n/(n-2) * Lower.ini
ELRtest.tmp <- cal.weight.JEL(matrix(g.vec,,1))$ELR
if (ELRtest.tmp <= level) stop()

# Upper limit
iter <- 0
dif <- 1
t1 <- MLE
t2 <- Upper.ini 
while(dif>tol & iter<maxit){
  tavg <- (t1+t2)/2
  g.vec <- J.vec - c(rep((n0-1)/n1,n1),rep((n1-1)/n0,n0)) * n/(n-2) * tavg
  elratio <- cal.weight.JEL(matrix(g.vec,,1))$ELR
  if(elratio > level) t2 <- tavg
  if(elratio <= level) t1 <- tavg
  dif <- t2-t1
  iter <- iter+1
}
UB <- (t1+t2)/2

# Lower limit
iter <- 0
dif <- 1
t1 <- MLE
t2 <- Lower.ini 
while(dif>tol & iter<maxit){
  tavg <- (t1+t2)/2
  g.vec <- J.vec - c(rep((n0-1)/n1,n1),rep((n1-1)/n0,n0)) * n/(n-2) * tavg
  elratio <- cal.weight.JEL(matrix(g.vec,,1))$ELR
  if(elratio > level) t2 <- tavg
  if(elratio <= level) t1 <- tavg
  dif <- t1-t2
  iter <- iter+1
}
LB <- (t1+t2)/2

cover <- ((LB<del0)&(UB>del0))
return(cover)
}

# Restricted net change of longer survival:                               #
# P(min(T1,\tau) > m + min(T0,\tau)) - P(min(T0,\tau) > m + min(T1,\tau)) #
findUL.PP <- function(maxit = 30, Lower.ini = -1, Upper.ini = 1, Y1, Y0, p1, p0, d1, d0, m = 0, MLE, level, del0, tol = 0.001){
n1 <- length(Y1)
n0 <- length(Y0)
n <- n1 + n0
J.vec <- Jack.PP(Y1 = Y1, Y0 = Y0, p1 = p1, p0 = p0, d1 = d1, d0 = d0, m = m)
g.vec <- J.vec - c(rep((n0-1)/n1,n1),rep((n1-1)/n0,n0)) * n/(n-2) * Upper.ini
ELRtest.tmp <- cal.weight.JEL(matrix(g.vec,,1))$ELR
if (ELRtest.tmp <= level) stop()
g.vec <- J.vec - c(rep((n0-1)/n1,n1),rep((n1-1)/n0,n0)) * n/(n-2) * Lower.ini
ELRtest.tmp <- cal.weight.JEL(matrix(g.vec,,1))$ELR
if (ELRtest.tmp <= level) stop()

# Upper limit
iter <- 0
dif <- 1
t1 <- MLE
t2 <- Upper.ini 
while(dif>tol & iter<maxit){
  tavg <- (t1+t2)/2
  g.vec <- J.vec - c(rep((n0-1)/n1,n1),rep((n1-1)/n0,n0)) * n/(n-2) * tavg
  elratio <- cal.weight.JEL(matrix(g.vec,,1))$ELR
  if(elratio > level) t2 <- tavg
  if(elratio <= level) t1 <- tavg
  dif <- t2-t1
  iter <- iter+1
}
UB <- (t1+t2)/2

# Lower limit
iter <- 0
dif <- 1
t1 <- MLE
t2 <- Lower.ini 
while(dif>tol & iter<maxit){
  tavg <- (t1+t2)/2
  g.vec <- J.vec - c(rep((n0-1)/n1,n1),rep((n1-1)/n0,n0)) * n/(n-2) * tavg
  elratio <- cal.weight.JEL(matrix(g.vec,,1))$ELR
  if(elratio > level) t2 <- tavg
  if(elratio <= level) t1 <- tavg
  dif <- t1-t2
  iter <- iter+1
}
LB <- (t1+t2)/2

cover <- ((LB<del0)&(UB>del0))
return(cover)
}


########################################################
# Caculate the standard error for the IPCW U-statistic #
# and the critical value of the empirical likelihood   #
# ratio test using bootstrap with parallel computing   # 
########################################################

# Bootstrap samples to be run simultaneously on one core #
Boot.cl <- function(corenum,odata,delhat,b.num.cl){
boot.iter <- 1
quan <- matrix(0,12,b.num.cl)
while (boot.iter<=b.num.cl){
tryCatch({
n <- nrow(odata)
idx <- sample(1:n, n, replace = TRUE, prob = NULL)
newodat <- as.data.frame(odata[idx,])
A <- newodat$A
d <- newodat$d

while ((prod(A==1)==1)|(prod(A==0)==1)|(prod(d[A==1]==0)==1)|(prod(d[A==1]==1)==1)|(prod(d[A==0]==0)==1)|(prod(d[A==0]==1)==1)){
idx <- sample(1:n, n, replace = TRUE, prob = NULL)
newodat <- as.data.frame(odata[idx,])
A <- newodat$A
d <- newodat$d
}

Y <- newodat$Y
X <- newodat$X
S1 <- c(1:n)[(A==1)]
S0 <- c(1:n)[(A==0)]
S11 <- c(1:n)[(A==1 & d==1)]
S01 <- c(1:n)[(A==0 & d==1)]
n1 <- sum(A)
n0 <- n - n1
n11 <- sum(A==1 & d==1)
n01 <- sum(A==0 & d==1)
Y1 <- Y[S1]
Y0 <- Y[S0]
Y1.cc <- Y[S11]
Y0.cc <- Y[S01]
d1 <- d[S1]
d0 <- d[S0]
X1 <- X[S1]
X0 <- X[S0]

p1 <- cal_KM(Y1,1-d1,Y1)
p0 <- cal_KM(Y0,1-d0,Y0)
if (any(p1 <= 1E-5) | any(p0 <= 1E-5)) stop()
p1.cc <- p1[match(S11,S1)]
p0.cc <- p0[match(S01,S0)]

quan[1,boot.iter] <- mean(apply(matrix(Y1.cc,1,),2,"-",Y0.cc)) 
quan[2,boot.iter] <- sum(apply(matrix(Y1.cc,1,),2,"-",Y0.cc)/apply(matrix(p1.cc,1,),2,"*",p0.cc))/(n1*n0)
J.vec <- Jack.RMST(Y1 = Y1, Y0 = Y0, p1 = p1, p0 = p0, d1 = d1, d0 = d0)
g.vec <- J.vec - c(rep((n0-1)/n1,n1),rep((n1-1)/n0,n0)) * n/(n-2) * delhat[1]
quan[3,boot.iter] <- cal.weight.JEL(matrix(g.vec,,1))$ELR

quan[4,boot.iter] <- mean(apply(matrix(Y1.cc,1,),2,">",Y0.cc * 1.1)) 
quan[5,boot.iter] <- sum(apply(matrix(Y1.cc,1,),2,">",Y0.cc*1.1)/apply(matrix(p1.cc,1,),2,"*",p0.cc))/(n1*n0)
J.vec <- Jack.ratio(Y1 = Y1, Y0 = Y0, p1 = p1, p0 = p0, d1 = d1, d0 = d0, m = 1.1)
g.vec <- J.vec - c(rep((n0-1)/n1,n1),rep((n1-1)/n0,n0)) * n/(n-2) * delhat[2]
quan[6,boot.iter] <- cal.weight.JEL(matrix(g.vec,,1))$ELR

quan[7,boot.iter] <- mean(apply(matrix(Y1.cc,1,),2,">",Y0.cc))-mean(apply(matrix(Y0.cc,1,),2,">",Y1.cc)) 
quan[8,boot.iter] <- sum(apply(matrix(Y1.cc,1,),2,">",Y0.cc)/apply(matrix(p1.cc,1,),2,"*",p0.cc))/(n1*n0)
J.vec <- Jack.dif(Y1 = Y1, Y0 = Y0, p1 = p1, p0 = p0, d1 = d1, d0 = d0, m = 0)
g.vec <- J.vec - c(rep((n0-1)/n1,n1),rep((n1-1)/n0,n0)) * n/(n-2) * delhat[3]
quan[9,boot.iter] <- cal.weight.JEL(matrix(g.vec,,1))$ELR

quan[10,boot.iter] <- mean(apply(matrix(Y1.cc,1,),2,">",Y0.cc))-mean(apply(matrix(Y0.cc,1,),2,">",Y1.cc)) 
quan[11,boot.iter] <- sum((apply(matrix(Y1.cc,1,),2,">",Y0.cc)-t(apply(matrix(Y0.cc,1,),2,">",Y1.cc)))/apply(matrix(p1.cc,1,),2,"*",p0.cc))/(n1*n0)
J.vec <- Jack.PP(Y1 = Y1, Y0 = Y0, p1 = p1, p0 = p0, d1 = d1, d0 = d0, m = 0)
g.vec <- J.vec - c(rep((n0-1)/n1,n1),rep((n1-1)/n0,n0)) * n/(n-2) * delhat[4]
quan[12,boot.iter] <- cal.weight.JEL(matrix(g.vec,,1))$ELR

boot.iter <- boot.iter+1
},error = function(e){})
}
  return(quan)
}

# Combine all the bootstrap results #
Boot <- function(odata,delhat){
b.num <- 500
cl.num <- 10
b.est <- do.call(cbind, mclapply(seq_len(cl.num), Boot.cl, 
  mc.cores = cl.num, odata = odata, delhat = delhat, b.num.cl = b.num/cl.num))
return(list(
se.cc.RMST  = sd(b.est[1,]),  se.im.RMST  = sd(b.est[2,]),  avg.RMST  = mean(b.est[3,]), 
se.cc.ratio = sd(b.est[4,]),  se.im.ratio = sd(b.est[5,]),  avg.ratio = mean(b.est[6,]),  
se.cc.dif   = sd(b.est[7,]),  se.im.dif   = sd(b.est[8,]),  avg.dif   = mean(b.est[9,]), 
se.cc.PP    = sd(b.est[10,]), se.im.PP    = sd(b.est[11,]), avg.PP    = mean(b.est[12,])))
}
